/*

    GlobalSat DG-100 GPS data logger download.

    Copyright (C) 2007  Mirko Parthey, mirko.parthey@informatik.tu-chemnitz.de
    Copyright (C) 2005-2008  Robert Lipe, robertlipe@gpsbabel.org
    Copyright (C) 2009  Paul Fleischer, paul@xpg.dk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA

 */
#ifndef _DG100_COMM_H
#define _DG100_COMM_H

#include "defs.h"
#include <stdbool.h>

/* maximum frame size observed so far: 1817 bytes
 *   (dg100cmd_getfileheader returning 150 entries)
 * dg100cmd_getfileheader is the only answer type of variable length,
 * answers of other types are always shorter than 1817 bytes */
#define FRAME_MAXLEN 4096

enum dg100_command_id {
	dg100cmd_getconfig     = 0xB7,
	dg100cmd_setconfig     = 0xB8,
	dg100cmd_getfileheader = 0xBB,
	dg100cmd_getfile       = 0xB5,
	dg100cmd_erase         = 0xBA,
	dg100cmd_getid         = 0xBF,
	dg100cmd_setid         = 0xC0,
	dg100cmd_gpsmouse      = 0xBC
};

struct dg100_command {
	int  id;
	int  sendsize;
	int  recvsize;
	int  trailing_bytes;
	const char *text;	/* Textual description for debugging */
};

struct dynarray16 {
	unsigned count; /* number of elements used */
	unsigned limit; /* number of elements allocated */
	gbint16 *data;
};

#define THRESHOLD_DISABLE 0
#define THRESHOLD_ENABLE 1

enum log_type {
  li_time = 0,
  li_distance = 1
};
struct loginterval {
        enum log_type flag;
	gbuint32 time;
	gbuint32 dist;
};

enum flag {
  fl_disable = 0,
  fl_enable = 1
};

enum infotype {
  it_pos = 0,
  it_pos_speed_time = 1,
  it_pos_speed_time_alt = 2
};

struct dg100config {
        enum infotype infotype;
	enum flag spd_thresh_flag; 
	gbuint32 spd_thresh; 
	enum flag dist_thresh_flag; 
	gbuint32 dist_thresh; 
	struct loginterval loginterval[3]; 
	gbuint8 memfree; 
        gbuint8 waas_flag;
};


void dg100_rd_deinit(void);
bool dg100_rd_init(const char *fname);

int dg100_erase();
int dg100_request(gbuint8 cmd, const void *sendbuf, void *recvbuf, size_t count);

int dg100_recv(gbuint8 expected_id, void *buf, unsigned int len);

void dg100_decodeconfig(struct dg100config *conf, const gbuint8 *data);
void dg100_encodeconfig(gbuint8 *data, const struct dg100config *conf);
void dg100_set_gpsmouse(bool enable);

void dg100_get_config(struct dg100config *conf);
void dg100_set_config(struct dg100config *conf);


#endif
